import _initializerDefineProperty from "@babel/runtime/helpers/initializerDefineProperty";
import _applyDecoratedDescriptor from "@babel/runtime/helpers/applyDecoratedDescriptor";
import _initializerWarningHelper from "@babel/runtime/helpers/initializerWarningHelper";

var _dec, _dec2, _dec3, _dec4, _dec5, _dec6, _dec7, _class, _class2, _descriptor, _descriptor2, _descriptor3, _descriptor4, _descriptor5, _descriptor6, _class3, _temp;

import { CanvasConfig, ContextService, Renderable, RenderingContext, getEuler, fromRotationTranslationScale } from '@antv/g';
import { isArray } from '@antv/util';
import { inject, injectable } from 'inversify';
import { vec3 } from 'gl-matrix';
import { PathGeneratorFactory } from './shapes/paths';
import { StyleRendererFactory } from './shapes/styles';
import { StyleParser } from './shapes/StyleParser';
var SHAPE_ATTRS_MAP = {
  fill: 'fillStyle',
  stroke: 'strokeStyle',
  opacity: 'globalAlpha'
};
export var CanvasRendererPlugin = (_dec = injectable(), _dec2 = inject(CanvasConfig), _dec3 = inject(ContextService), _dec4 = inject(RenderingContext), _dec5 = inject(StyleParser), _dec6 = inject(PathGeneratorFactory), _dec7 = inject(StyleRendererFactory), _dec(_class = (_class2 = (_temp = _class3 = /*#__PURE__*/function () {
  function CanvasRendererPlugin() {
    _initializerDefineProperty(this, "canvasConfig", _descriptor, this);

    _initializerDefineProperty(this, "contextService", _descriptor2, this);

    _initializerDefineProperty(this, "renderingContext", _descriptor3, this);

    _initializerDefineProperty(this, "styleParser", _descriptor4, this);

    _initializerDefineProperty(this, "pathGeneratorFactory", _descriptor5, this);

    _initializerDefineProperty(this, "styleRendererFactory", _descriptor6, this);

    this.lastDirtyRectangle = void 0;
  }

  var _proto = CanvasRendererPlugin.prototype;

  _proto.apply = function apply(renderingService) {
    var _this = this;

    renderingService.hooks.beforeRender.tap(CanvasRendererPlugin.tag, function () {
      var _this$canvasConfig;

      var context = _this.contextService.getContext();

      var _this$canvasConfig$re = (_this$canvasConfig = _this.canvasConfig) === null || _this$canvasConfig === void 0 ? void 0 : _this$canvasConfig.renderer.getConfig(),
          enableDirtyRectangleRendering = _this$canvasConfig$re.enableDirtyRectangleRendering,
          enableDirtyRectangleRenderingDebug = _this$canvasConfig$re.enableDirtyRectangleRenderingDebug;

      var dirtyAABB = _this.renderingContext.dirtyRectangle;

      if (context) {
        if (!enableDirtyRectangleRendering) {
          context.clearRect(0, 0, _this.canvasConfig.width, _this.canvasConfig.height);
          context.save();
        } else {
          var dirtyRectangle = _this.convertAABB2Rect(dirtyAABB);

          context.clearRect(dirtyRectangle.x, dirtyRectangle.y, dirtyRectangle.width, dirtyRectangle.height);

          if (enableDirtyRectangleRenderingDebug) {
            if (_this.lastDirtyRectangle) {
              context.clearRect(_this.lastDirtyRectangle.x, _this.lastDirtyRectangle.y, _this.lastDirtyRectangle.width, _this.lastDirtyRectangle.height);
            }
          } // clip dirty rectangle


          context.save();
          context.beginPath();
          context.rect(dirtyRectangle.x, dirtyRectangle.y, dirtyRectangle.width, dirtyRectangle.height);
          context.clip(); // draw dirty rectangle on DEBUG mode

          if (enableDirtyRectangleRenderingDebug) {
            _this.drawDirtyRectangle(context, dirtyRectangle);

            _this.lastDirtyRectangle = dirtyRectangle;
          }
        }
      }
    });
    renderingService.hooks.afterRender.tap(CanvasRendererPlugin.tag, function () {
      var context = _this.contextService.getContext();

      context.restore();
    });
    renderingService.hooks.render.tap(CanvasRendererPlugin.tag, function (objects) {
      var context = _this.contextService.getContext();

      objects.forEach(function (object) {
        var nodeType = object.nodeType; // reset transformation

        context.save();
        var originMatrix = context.getTransform(); // apply RTS transformation in world space

        _this.applyTransform(context, object); // apply attributes to context


        _this.applyAttributesToContext(context, object.attributes); // generate path in local space


        var generatePath = _this.pathGeneratorFactory(nodeType);

        if (generatePath) {
          generatePath(context, object.attributes);
        } // fill & stroke


        var styleRenderer = _this.styleRendererFactory(nodeType);

        if (styleRenderer) {
          styleRenderer.render(context, object.attributes);
        }

        context.setTransform(originMatrix); // finish rendering, clear dirty flag

        var renderable = object.getEntity().getComponent(Renderable);
        renderable.dirty = false;
        context.restore();
      });
    });
  };

  _proto.convertAABB2Rect = function convertAABB2Rect(aabb) {
    var min = aabb.getMin();
    var max = aabb.getMax(); // expand the rectangle a bit to avoid artifacts
    // @see https://www.yuque.com/antv/ou292n/bi8nix#ExvCu

    var minX = Math.floor(min[0]);
    var minY = Math.floor(min[1]);
    var maxX = Math.ceil(max[0]);
    var maxY = Math.ceil(max[1]);
    var width = maxX - minX;
    var height = maxY - minY;
    return {
      x: minX,
      y: minY,
      width: width,
      height: height
    };
  };

  _proto.drawDirtyRectangle = function drawDirtyRectangle(context, _ref) {
    var x = _ref.x,
        y = _ref.y,
        width = _ref.width,
        height = _ref.height;
    context.beginPath();
    context.rect(x + 1, y + 1, width - 1, height - 1);
    context.closePath();
    context.lineWidth = 1;
    context.stroke();
  };

  _proto.applyTransform = function applyTransform(context, object) {
    // const { width, height } = this.canvasConfig;
    var _getEuler = getEuler(vec3.create(), object.getRotation()),
        ex = _getEuler[0],
        ey = _getEuler[1],
        ez = _getEuler[2];

    var _object$getPosition = object.getPosition(),
        x = _object$getPosition[0],
        y = _object$getPosition[1];

    var _object$getScale = object.getScale(),
        scaleX = _object$getScale[0],
        scaleY = _object$getScale[1]; // TODO: use MVP matrix
    // const viewMatrix = this.camera.getViewTransform()!;
    // const viewProjectionMatrix = mat4.multiply(
    //   mat4.create(),
    //   this.camera.getPerspective(),
    //   viewMatrix,
    // );
    // const modelMatrix = this.sceneGraphService.getWorldTransform(entity, entity.getComponent(Transform));
    // const mvpMatrix = mat4.multiply(
    //   mat4.create(),
    //   viewProjectionMatrix,
    //   modelMatrix,
    // );
    // const [tx, ty] = mat4.getTranslation(vec3.create(), mvpMatrix);
    // const [sx, sy] = mat4.getScaling(vec3.create(), mvpMatrix);
    // const rotation = mat4.getRotation(quat.create(), mvpMatrix);
    // const [eux, euy, euz] = getEuler(vec3.create(), rotation);
    // gimbal lock at 90 degrees


    var rts = fromRotationTranslationScale(ex || ez, x, y, scaleX, scaleY); // @see https://developer.mozilla.org/en-US/docs/Web/API/Canvas_API/Tutorial/Transformations

    context.transform(rts[0], rts[1], rts[3], rts[4], rts[6], rts[7]);
  };

  _proto.applyAttributesToContext = function applyAttributesToContext(context, attrs) {
    for (var k in attrs) {
      var v = attrs[k]; // 转换一下不与 canvas 兼容的属性名

      var name = SHAPE_ATTRS_MAP[k] ? SHAPE_ATTRS_MAP[k] : k;

      if (name === 'lineDash' && context.setLineDash) {
        isArray(v) && context.setLineDash(v);
      } else {
        if (name === 'strokeStyle' || name === 'fillStyle') {
          // 如果存在渐变、pattern 这个开销有些大
          // 可以考虑缓存机制，通过 hasUpdate 来避免一些运算
          v = this.styleParser.parse(v);
        } else if (name === 'globalAlpha') {
          // opacity 效果可以叠加，子元素的 opacity 需要与父元素 opacity 相乘
          v = v * context.globalAlpha;
        } // @ts-ignore


        context[name] = v;
      }
    }
  };

  return CanvasRendererPlugin;
}(), _class3.tag = 'CanvasRendererPlugin', _temp), (_descriptor = _applyDecoratedDescriptor(_class2.prototype, "canvasConfig", [_dec2], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor2 = _applyDecoratedDescriptor(_class2.prototype, "contextService", [_dec3], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor3 = _applyDecoratedDescriptor(_class2.prototype, "renderingContext", [_dec4], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor4 = _applyDecoratedDescriptor(_class2.prototype, "styleParser", [_dec5], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor5 = _applyDecoratedDescriptor(_class2.prototype, "pathGeneratorFactory", [_dec6], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor6 = _applyDecoratedDescriptor(_class2.prototype, "styleRendererFactory", [_dec7], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
})), _class2)) || _class);
//# sourceMappingURL=CanvasRendererPlugin.js.map