import getArcParams from '../../utils/arc-params';
export function generatePath(context, attributes) {
  var _attributes$x = attributes.x,
      x = _attributes$x === void 0 ? 0 : _attributes$x,
      _attributes$y = attributes.y,
      y = _attributes$y === void 0 ? 0 : _attributes$y,
      path = attributes.path,
      arcParamsCache = attributes.arcParamsCache;

  if (!arcParamsCache) {
    attributes.arcParamsCache = {};
  }

  var currentPoint = [-x, -y]; // 当前图形

  var startMovePoint = [-x, -y]; // 开始 M 的点，可能会有多个

  context.beginPath();

  for (var i = 0; i < path.length; i++) {
    var params = path[i]; // eg. M 100 200

    var command = params[0]; // V,H,S,T 都在前面被转换成标准形式

    switch (command) {
      case 'M':
        context.moveTo(params[1] - x, params[2] - y);
        startMovePoint = [params[1] - x, params[2] - y];
        break;

      case 'L':
        context.lineTo(params[1] - x, params[2] - y);
        break;

      case 'Q':
        context.quadraticCurveTo(params[1], params[2], params[3] - x, params[4] - y);
        break;

      case 'C':
        context.bezierCurveTo(params[1], params[2], params[3], params[4], params[5] - x, params[6] - y);
        break;

      case 'A':
        {
          var arcParams = void 0; // 为了加速绘制，可以提供参数的缓存，各个图形自己缓存

          if (arcParamsCache) {
            arcParams = arcParamsCache[i];

            if (!arcParams) {
              arcParams = getArcParams(currentPoint, params, x, y);
              arcParamsCache[i] = arcParams;
            }
          } else {
            arcParams = getArcParams(currentPoint, params, x, y);
          }

          var _arcParams = arcParams,
              cx = _arcParams.cx,
              cy = _arcParams.cy,
              rx = _arcParams.rx,
              ry = _arcParams.ry,
              startAngle = _arcParams.startAngle,
              endAngle = _arcParams.endAngle,
              xRotation = _arcParams.xRotation,
              sweepFlag = _arcParams.sweepFlag; // 直接使用椭圆的 api

          if (context.ellipse) {
            context.ellipse(cx, cy, rx, ry, xRotation, startAngle, endAngle, !!(1 - sweepFlag));
          } else {
            // 如果不支持，则使用圆来绘制，进行变形
            var r = rx > ry ? rx : ry;
            var scaleX = rx > ry ? 1 : rx / ry;
            var scaleY = rx > ry ? ry / rx : 1;
            context.translate(cx, cy);
            context.rotate(xRotation);
            context.scale(scaleX, scaleY);
            context.arc(0, 0, r, startAngle, endAngle, !!(1 - sweepFlag));
            context.scale(1 / scaleX, 1 / scaleY);
            context.rotate(-xRotation);
            context.translate(-cx, -cy);
          }

          break;
        }

      case 'Z':
        context.closePath();
        break;

      default:
        break;
    } // 有了 Z 后，当前节点从开始 M 的点开始


    if (command === 'Z') {
      currentPoint = startMovePoint;
    } else {
      var len = params.length;
      currentPoint = [params[len - 2] - x, params[len - 1] - y];
    }
  }
}
//# sourceMappingURL=Path.js.map