import _initializerDefineProperty from "@babel/runtime/helpers/initializerDefineProperty";
import _applyDecoratedDescriptor from "@babel/runtime/helpers/applyDecoratedDescriptor";
import _initializerWarningHelper from "@babel/runtime/helpers/initializerWarningHelper";

var _dec, _dec2, _class, _class2, _descriptor;

import { TextService } from '@antv/g';
import { inject, injectable } from 'inversify';
import { isNil } from '@antv/util';
export var TextRenderer = (_dec = injectable(), _dec2 = inject(TextService), _dec(_class = (_class2 = /*#__PURE__*/function () {
  function TextRenderer() {
    _initializerDefineProperty(this, "textService", _descriptor, this);
  }

  var _proto = TextRenderer.prototype;

  _proto.render = function render(context, attributes) {
    var _attributes$text = attributes.text,
        text = _attributes$text === void 0 ? '' : _attributes$text,
        _attributes$lineWidth = attributes.lineWidth,
        lineWidth = _attributes$lineWidth === void 0 ? 0 : _attributes$lineWidth,
        textAlign = attributes.textAlign,
        textBaseline = attributes.textBaseline,
        lineJoin = attributes.lineJoin,
        miterLimit = attributes.miterLimit,
        _attributes$padding = attributes.padding,
        padding = _attributes$padding === void 0 ? 0 : _attributes$padding,
        letterSpacing = attributes.letterSpacing,
        stroke = attributes.stroke,
        fill = attributes.fill,
        fillOpacity = attributes.fillOpacity,
        strokeOpacity = attributes.strokeOpacity,
        opacity = attributes.opacity;

    var _this$textService$mea = this.textService.measureText(text, attributes),
        font = _this$textService$mea.font,
        lines = _this$textService$mea.lines,
        height = _this$textService$mea.height,
        lineHeight = _this$textService$mea.lineHeight;

    context.font = font;
    context.lineWidth = lineWidth;
    context.textBaseline = textBaseline;
    context.lineJoin = lineJoin;
    context.miterLimit = miterLimit;
    var linePositionY = 0; // handle vertical text baseline

    if (textBaseline === 'middle') {
      linePositionY = -height / 2 - lineHeight / 2;
    } else if (textBaseline === 'bottom' || textBaseline === 'alphabetic' || textBaseline === 'ideographic') {
      linePositionY = -height;
    } else if (textBaseline === 'top' || textBaseline === 'hanging') {
      linePositionY = -lineHeight;
    } // draw lines line by line


    for (var i = 0; i < lines.length; i++) {
      var linePositionX = lineWidth / 2;
      linePositionY += lineHeight; // no need to re-position X, cause we already set text align
      // @see https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/textAlign

      if (!isNil(stroke) && lineWidth) {
        this.drawLetterSpacing(context, lines[i], linePositionX + padding, linePositionY + padding, letterSpacing, fillOpacity, strokeOpacity, opacity, true);
      }

      if (!isNil(fill)) {
        this.drawLetterSpacing(context, lines[i], linePositionX + padding, linePositionY + padding, letterSpacing, fillOpacity, strokeOpacity, opacity);
      }
    }
  };

  _proto.drawLetterSpacing = function drawLetterSpacing(context, text, x, y, letterSpacing, fillOpacity, strokeOpacity, opacity, isStroke) {
    if (isStroke === void 0) {
      isStroke = false;
    }

    // letterSpacing of 0 means normal
    if (letterSpacing === 0) {
      if (isStroke) {
        this.strokeText(context, text, x, y, strokeOpacity);
      } else {
        this.fillText(context, text, x, y, fillOpacity, opacity);
      }

      return;
    }

    var currentPosition = x; // Using Array.from correctly splits characters whilst keeping emoji together.
    // This is not supported on IE as it requires ES6, so regular text splitting occurs.
    // This also doesn't account for emoji that are multiple emoji put together to make something else.
    // Handling all of this would require a big library itself.
    // https://medium.com/@giltayar/iterating-over-emoji-characters-the-es6-way-f06e4589516
    // https://github.com/orling/grapheme-splitter

    var stringArray = Array.from ? Array.from(text) : text.split('');
    var previousWidth = context.measureText(text).width;
    var currentWidth = 0;

    for (var i = 0; i < stringArray.length; ++i) {
      var currentChar = stringArray[i];

      if (isStroke) {
        this.strokeText(context, currentChar, currentPosition, y, strokeOpacity);
      } else {
        this.fillText(context, currentChar, currentPosition, y, fillOpacity, opacity);
      }

      currentWidth = context.measureText(text.substring(i + 1)).width;
      currentPosition += previousWidth - currentWidth + letterSpacing;
      previousWidth = currentWidth;
    }
  };

  _proto.fillText = function fillText(context, text, x, y, fillOpacity, opacity) {
    var applyOpacity = !isNil(fillOpacity) && fillOpacity !== 1;

    if (applyOpacity) {
      context.globalAlpha = fillOpacity;
    }

    context.fillText(text, x, y);

    if (applyOpacity) {
      context.globalAlpha = opacity;
    }
  };

  _proto.strokeText = function strokeText(context, text, x, y, strokeOpacity) {
    var applyOpacity = !isNil(strokeOpacity) && strokeOpacity !== 1;

    if (applyOpacity) {
      context.globalAlpha = strokeOpacity;
    }

    context.strokeText(text, x, y);
  };

  return TextRenderer;
}(), (_descriptor = _applyDecoratedDescriptor(_class2.prototype, "textService", [_dec2], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
})), _class2)) || _class);
//# sourceMappingURL=Text.js.map