"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
exports.__esModule = true;
exports.CanvasRendererPlugin = void 0;
var g_1 = require("@antv/g");
var util_1 = require("@antv/util");
var inversify_1 = require("inversify");
var gl_matrix_1 = require("gl-matrix");
var paths_1 = require("./shapes/paths");
var styles_1 = require("./shapes/styles");
var StyleParser_1 = require("./shapes/StyleParser");
var SHAPE_ATTRS_MAP = {
    fill: 'fillStyle',
    stroke: 'strokeStyle',
    opacity: 'globalAlpha'
};
var CanvasRendererPlugin = /** @class */ (function () {
    function CanvasRendererPlugin() {
    }
    CanvasRendererPlugin_1 = CanvasRendererPlugin;
    CanvasRendererPlugin.prototype.apply = function (renderingService) {
        var _this = this;
        renderingService.hooks.beforeRender.tap(CanvasRendererPlugin_1.tag, function () {
            var _a;
            var context = _this.contextService.getContext();
            var _b = (_a = _this.canvasConfig) === null || _a === void 0 ? void 0 : _a.renderer.getConfig(), enableDirtyRectangleRendering = _b.enableDirtyRectangleRendering, enableDirtyRectangleRenderingDebug = _b.enableDirtyRectangleRenderingDebug;
            var dirtyAABB = _this.renderingContext.dirtyRectangle;
            if (context) {
                if (!enableDirtyRectangleRendering) {
                    context.clearRect(0, 0, _this.canvasConfig.width, _this.canvasConfig.height);
                    context.save();
                }
                else {
                    var dirtyRectangle = _this.convertAABB2Rect(dirtyAABB);
                    context.clearRect(dirtyRectangle.x, dirtyRectangle.y, dirtyRectangle.width, dirtyRectangle.height);
                    if (enableDirtyRectangleRenderingDebug) {
                        if (_this.lastDirtyRectangle) {
                            context.clearRect(_this.lastDirtyRectangle.x, _this.lastDirtyRectangle.y, _this.lastDirtyRectangle.width, _this.lastDirtyRectangle.height);
                        }
                    }
                    // clip dirty rectangle
                    context.save();
                    context.beginPath();
                    context.rect(dirtyRectangle.x, dirtyRectangle.y, dirtyRectangle.width, dirtyRectangle.height);
                    context.clip();
                    // draw dirty rectangle on DEBUG mode
                    if (enableDirtyRectangleRenderingDebug) {
                        _this.drawDirtyRectangle(context, dirtyRectangle);
                        _this.lastDirtyRectangle = dirtyRectangle;
                    }
                }
            }
        });
        renderingService.hooks.afterRender.tap(CanvasRendererPlugin_1.tag, function () {
            var context = _this.contextService.getContext();
            context.restore();
        });
        renderingService.hooks.render.tap(CanvasRendererPlugin_1.tag, function (objects) {
            var context = _this.contextService.getContext();
            objects.forEach(function (object) {
                var nodeType = object.nodeType;
                // reset transformation
                context.save();
                var originMatrix = context.getTransform();
                // apply RTS transformation in world space
                _this.applyTransform(context, object);
                // apply attributes to context
                _this.applyAttributesToContext(context, object.attributes);
                // generate path in local space
                var generatePath = _this.pathGeneratorFactory(nodeType);
                if (generatePath) {
                    generatePath(context, object.attributes);
                }
                // fill & stroke
                var styleRenderer = _this.styleRendererFactory(nodeType);
                if (styleRenderer) {
                    styleRenderer.render(context, object.attributes);
                }
                context.setTransform(originMatrix);
                // finish rendering, clear dirty flag
                var renderable = object.getEntity().getComponent(g_1.Renderable);
                renderable.dirty = false;
                context.restore();
            });
        });
    };
    CanvasRendererPlugin.prototype.convertAABB2Rect = function (aabb) {
        var min = aabb.getMin();
        var max = aabb.getMax();
        // expand the rectangle a bit to avoid artifacts
        // @see https://www.yuque.com/antv/ou292n/bi8nix#ExvCu
        var minX = Math.floor(min[0]);
        var minY = Math.floor(min[1]);
        var maxX = Math.ceil(max[0]);
        var maxY = Math.ceil(max[1]);
        var width = maxX - minX;
        var height = maxY - minY;
        return { x: minX, y: minY, width: width, height: height };
    };
    CanvasRendererPlugin.prototype.drawDirtyRectangle = function (context, _a) {
        var x = _a.x, y = _a.y, width = _a.width, height = _a.height;
        context.beginPath();
        context.rect(x + 1, y + 1, width - 1, height - 1);
        context.closePath();
        context.lineWidth = 1;
        context.stroke();
    };
    CanvasRendererPlugin.prototype.applyTransform = function (context, object) {
        // const { width, height } = this.canvasConfig;
        var _a = g_1.getEuler(gl_matrix_1.vec3.create(), object.getRotation()), ex = _a[0], ey = _a[1], ez = _a[2];
        var _b = object.getPosition(), x = _b[0], y = _b[1];
        var _c = object.getScale(), scaleX = _c[0], scaleY = _c[1];
        // TODO: use MVP matrix
        // const viewMatrix = this.camera.getViewTransform()!;
        // const viewProjectionMatrix = mat4.multiply(
        //   mat4.create(),
        //   this.camera.getPerspective(),
        //   viewMatrix,
        // );
        // const modelMatrix = this.sceneGraphService.getWorldTransform(entity, entity.getComponent(Transform));
        // const mvpMatrix = mat4.multiply(
        //   mat4.create(),
        //   viewProjectionMatrix,
        //   modelMatrix,
        // );
        // const [tx, ty] = mat4.getTranslation(vec3.create(), mvpMatrix);
        // const [sx, sy] = mat4.getScaling(vec3.create(), mvpMatrix);
        // const rotation = mat4.getRotation(quat.create(), mvpMatrix);
        // const [eux, euy, euz] = getEuler(vec3.create(), rotation);
        // gimbal lock at 90 degrees
        var rts = g_1.fromRotationTranslationScale(ex || ez, x, y, scaleX, scaleY);
        // @see https://developer.mozilla.org/en-US/docs/Web/API/Canvas_API/Tutorial/Transformations
        context.transform(rts[0], rts[1], rts[3], rts[4], rts[6], rts[7]);
    };
    CanvasRendererPlugin.prototype.applyAttributesToContext = function (context, attrs) {
        for (var k in attrs) {
            var v = attrs[k];
            // 转换一下不与 canvas 兼容的属性名
            var name_1 = SHAPE_ATTRS_MAP[k] ? SHAPE_ATTRS_MAP[k] : k;
            if (name_1 === 'lineDash' && context.setLineDash) {
                util_1.isArray(v) && context.setLineDash(v);
            }
            else {
                if (name_1 === 'strokeStyle' || name_1 === 'fillStyle') {
                    // 如果存在渐变、pattern 这个开销有些大
                    // 可以考虑缓存机制，通过 hasUpdate 来避免一些运算
                    v = this.styleParser.parse(v);
                }
                else if (name_1 === 'globalAlpha') {
                    // opacity 效果可以叠加，子元素的 opacity 需要与父元素 opacity 相乘
                    v = v * context.globalAlpha;
                }
                // @ts-ignore
                context[name_1] = v;
            }
        }
    };
    var CanvasRendererPlugin_1;
    CanvasRendererPlugin.tag = 'CanvasRendererPlugin';
    __decorate([
        inversify_1.inject(g_1.CanvasConfig)
    ], CanvasRendererPlugin.prototype, "canvasConfig");
    __decorate([
        inversify_1.inject(g_1.ContextService)
    ], CanvasRendererPlugin.prototype, "contextService");
    __decorate([
        inversify_1.inject(g_1.RenderingContext)
    ], CanvasRendererPlugin.prototype, "renderingContext");
    __decorate([
        inversify_1.inject(StyleParser_1.StyleParser)
    ], CanvasRendererPlugin.prototype, "styleParser");
    __decorate([
        inversify_1.inject(paths_1.PathGeneratorFactory)
    ], CanvasRendererPlugin.prototype, "pathGeneratorFactory");
    __decorate([
        inversify_1.inject(styles_1.StyleRendererFactory)
    ], CanvasRendererPlugin.prototype, "styleRendererFactory");
    CanvasRendererPlugin = CanvasRendererPlugin_1 = __decorate([
        inversify_1.injectable()
    ], CanvasRendererPlugin);
    return CanvasRendererPlugin;
}());
exports.CanvasRendererPlugin = CanvasRendererPlugin;
