"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
exports.__esModule = true;
exports.TextRenderer = void 0;
var g_1 = require("@antv/g");
var inversify_1 = require("inversify");
var util_1 = require("@antv/util");
var TextRenderer = /** @class */ (function () {
    function TextRenderer() {
    }
    TextRenderer.prototype.render = function (context, attributes) {
        var _a = attributes.text, text = _a === void 0 ? '' : _a, _b = attributes.lineWidth, lineWidth = _b === void 0 ? 0 : _b, textAlign = attributes.textAlign, textBaseline = attributes.textBaseline, lineJoin = attributes.lineJoin, miterLimit = attributes.miterLimit, _c = attributes.padding, padding = _c === void 0 ? 0 : _c, letterSpacing = attributes.letterSpacing, stroke = attributes.stroke, fill = attributes.fill, fillOpacity = attributes.fillOpacity, strokeOpacity = attributes.strokeOpacity, opacity = attributes.opacity;
        var _d = this.textService.measureText(text, attributes), font = _d.font, lines = _d.lines, height = _d.height, lineHeight = _d.lineHeight;
        context.font = font;
        context.lineWidth = lineWidth;
        context.textBaseline = textBaseline;
        context.lineJoin = lineJoin;
        context.miterLimit = miterLimit;
        var linePositionY = 0;
        // handle vertical text baseline
        if (textBaseline === 'middle') {
            linePositionY = -height / 2 - lineHeight / 2;
        }
        else if (textBaseline === 'bottom' || textBaseline === 'alphabetic' || textBaseline === 'ideographic') {
            linePositionY = -height;
        }
        else if (textBaseline === 'top' || textBaseline === 'hanging') {
            linePositionY = -lineHeight;
        }
        // draw lines line by line
        for (var i = 0; i < lines.length; i++) {
            var linePositionX = lineWidth / 2;
            linePositionY += lineHeight;
            // no need to re-position X, cause we already set text align
            // @see https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/textAlign
            if (!util_1.isNil(stroke) && lineWidth) {
                this.drawLetterSpacing(context, lines[i], linePositionX + padding, linePositionY + padding, letterSpacing, fillOpacity, strokeOpacity, opacity, true);
            }
            if (!util_1.isNil(fill)) {
                this.drawLetterSpacing(context, lines[i], linePositionX + padding, linePositionY + padding, letterSpacing, fillOpacity, strokeOpacity, opacity);
            }
        }
    };
    TextRenderer.prototype.drawLetterSpacing = function (context, text, x, y, letterSpacing, fillOpacity, strokeOpacity, opacity, isStroke) {
        if (isStroke === void 0) { isStroke = false; }
        // letterSpacing of 0 means normal
        if (letterSpacing === 0) {
            if (isStroke) {
                this.strokeText(context, text, x, y, strokeOpacity);
            }
            else {
                this.fillText(context, text, x, y, fillOpacity, opacity);
            }
            return;
        }
        var currentPosition = x;
        // Using Array.from correctly splits characters whilst keeping emoji together.
        // This is not supported on IE as it requires ES6, so regular text splitting occurs.
        // This also doesn't account for emoji that are multiple emoji put together to make something else.
        // Handling all of this would require a big library itself.
        // https://medium.com/@giltayar/iterating-over-emoji-characters-the-es6-way-f06e4589516
        // https://github.com/orling/grapheme-splitter
        var stringArray = Array.from ? Array.from(text) : text.split('');
        var previousWidth = context.measureText(text).width;
        var currentWidth = 0;
        for (var i = 0; i < stringArray.length; ++i) {
            var currentChar = stringArray[i];
            if (isStroke) {
                this.strokeText(context, currentChar, currentPosition, y, strokeOpacity);
            }
            else {
                this.fillText(context, currentChar, currentPosition, y, fillOpacity, opacity);
            }
            currentWidth = context.measureText(text.substring(i + 1)).width;
            currentPosition += previousWidth - currentWidth + letterSpacing;
            previousWidth = currentWidth;
        }
    };
    TextRenderer.prototype.fillText = function (context, text, x, y, fillOpacity, opacity) {
        var applyOpacity = !util_1.isNil(fillOpacity) && fillOpacity !== 1;
        if (applyOpacity) {
            context.globalAlpha = fillOpacity;
        }
        context.fillText(text, x, y);
        if (applyOpacity) {
            context.globalAlpha = opacity;
        }
    };
    TextRenderer.prototype.strokeText = function (context, text, x, y, strokeOpacity) {
        var applyOpacity = !util_1.isNil(strokeOpacity) && strokeOpacity !== 1;
        if (applyOpacity) {
            context.globalAlpha = strokeOpacity;
        }
        context.strokeText(text, x, y);
    };
    __decorate([
        inversify_1.inject(g_1.TextService)
    ], TextRenderer.prototype, "textService");
    TextRenderer = __decorate([
        inversify_1.injectable()
    ], TextRenderer);
    return TextRenderer;
}());
exports.TextRenderer = TextRenderer;
